@@ -971,18 +971,17 @@ class NumerousClient:
 
         self._base_client.WriteDataList(to_indexed_data_list(data_writer()))
 
-    def _estimate_block_size(self, data):
-        return sum(len(json.dumps(value)) / 1e6 for value in data.values())
-
     def _split(self, data, parts: int):
+        """Keep numpy.array_split() logic."""
         length = len(data)
 
         size = length // parts + 1
-        for i in range(0, length % parts, size):
+        rest_start = (length % parts) * size
+        for i in range(0, rest_start, size):
             yield data[i:i + size]
 
         size = length // parts
-        for i in range(length % parts, length, size):
+        for i in range(rest_start, length, size):
             yield data[i:i + size]
 
     def _rows_to_blocks(self, rows):
@@ -990,10 +989,10 @@ class NumerousClient:
             return []
 
         data = {k: [float(r[k]) for r in rows] for k in rows[0]}
-        estimated_size = self._estimate_block_size(data)
-        maximum_block_size = 3.5  # allow some overhead
+        estimated_size = sum(len(json.dumps(value)) for value in data.values())
+        maximum_block_size = 3.5e6  # allow some overhead
         estimated_chunks = int(math.ceil(estimated_size / maximum_block_size))
-        blocks = [list() for i in range(estimated_chunks+1)]
+        blocks = [list() for _ in range(estimated_chunks)]
 
         # split data into chunks of equal size (except last chunk)
         for tag, values in data.items():
@@ -1285,7 +1284,6 @@ class NumerousClient:
         class _WriteDataStream(RequestResponseStream):
             def __init__(stream, client: NumerousClient):
                 super(_WriteDataStream, stream).__init__(
-                    client._base_client,
                     client._base_client.WriteDataStream,
                 )
 
@@ -1321,7 +1319,6 @@ class NumerousClient:
         class _ReadDataStream(RequestResponseStream):
             def __init__(stream, client: NumerousClient):
                 super(_ReadDataStream, stream).__init__(
-                    client._base_client,
                     client._base_client.ReadDataStream,
                 )
 
